/*------------------------------------------------------------------------
  File    : _rtb.p 
  Purpose : Roundtable Startup Procedure
  ----------------------------------------------------------------------*/
{adecomm/appserv.i}
{rtb/i/rtb_super.i}
{rtb/i/rtb_events.i}
{rtb/i/rtb_appserv.i}

DEFINE VARIABLE cError       AS CHARACTER NO-UNDO.
DEFINE VARIABLE cRtbPropath  AS CHARACTER NO-UNDO.
DEFINE VARIABLE cDlcPropath  AS CHARACTER NO-UNDO.
DEFINE VARIABLE cStartupProc AS CHARACTER NO-UNDO.
DEFINE VARIABLE hEvents      AS HANDLE    NO-UNDO.
DEFINE VARIABLE hFunc        AS HANDLE    NO-UNDO.
DEFINE VARIABLE hWinApi      AS HANDLE    NO-UNDO.
DEFINE VARIABLE hDesk        AS HANDLE    NO-UNDO.
DEFINE VARIABLE hWindow      AS HANDLE    NO-UNDO.
DEFINE VARIABLE iSid         AS INTEGER   NO-UNDO.
DEFINE VARIABLE lOk          AS LOGICAL   NO-UNDO INITIAL TRUE.

FUNCTION setProgressPropath   RETURNS LOGICAL (INPUT Pdlcpath AS CHARACTER) FORWARD.
FUNCTION setRoundtablePropath RETURNS CHARACTER()                           FORWARD.
FUNCTION isRoundtableRunning  RETURNS LOGICAL()                             FORWARD.

/*
 Save the default PROPATH before we mangle it all to hell..
*/
cDlcPropath = PROPATH.

/*
 Set the PROPATH based on where this program was started and return the value
*/
cRtbPropath = setRoundtablePropath().

/*
 Login to RTB..
*/
RUN rtb/w/rtb_lgin.w (cRtbPropath, OUTPUT cError, OUTPUT iSid).

IF cError <> "" THEN DO:
  IF NOT isRoundtableRunning() THEN
    QUIT.  
END.

/*
 Check for scenario where user is connected via AppServer but also has an 
 incorrect shortcut that connects to the RTB database.
*/
IF rtbGetServiceHandle() <> SESSION:HANDLE AND CONNECTED("rtb") THEN DO:
  
  MESSAGE
    "** Configuration appears to be invalid!" SKIP(1)
    "** You are connected to a Roundtable AppServer while also locally connected to the Roundtable database."
    "This can produce unpedictable results." SKIP(1)
    "Disconnect Roundtable database and continue?"
    VIEW-AS ALERT-BOX WARNING
    BUTTONS YES-NO
    TITLE "Roundtable"
    UPDATE lContinue AS LOGICAL.
  
  IF NOT lContinue THEN DO:
    RUN rtb_logout(iSid).
    RUN rtb_end_session.
  END.
    
  DISCONNECT rtb. /* disconnect for them if they continue */
  
END.

/*
 Run the Tabletop and PSC Desktop..
*/
SESSION:SET-WAIT-STATE("GENERAL").

DISPLAY
  SKIP(1)
  "Logging into Roundtable and initializing desktop..."
  SKIP(1)
  WITH FRAME frConnect
  OVERLAY
  CENTERED
  VIEW-AS DIALOG-BOX THREE-D
  TITLE "Roundtable".

/*
 Start events handler
*/
IF CONNECTED("rtb":U) THEN
  hEvents = fnRtbStartSuperProc("rtb_events.p",SESSION).
ELSE
  hEvents = fnRtbStartSuperProc("rtb_events_cl.p",SESSION).
  
PUBLISH {&RTB-USER-EVENT}
  (INPUT {&SESSION-STARTUP},
   INPUT STRING(iSid),
   INPUT "",
   INPUT "",
   OUTPUT lOk).

IF lOk THEN DO:
    
  /*
   Start the Roundtable Desktop
  */
  RUN rtb/w/rtb_desk.w PERSISTENT SET hDesk (iSid).
  
  /*
   Store the default Progress PROPATH
  */
  setProgressPropath(cDlcPropath).    

  IF NOT isRoundtableRunning() THEN DO:
      
    /*
     Run libraries and add to super procedure stack..
    */
    ASSIGN
      hFunc   = fnRtbStartSuperProc("rtb/p/rtb_funclib.p",SESSION)
      hWinApi = fnRtbStartSuperProc("rtb/p/rtb_winapi.p",SESSION).
  
    RUN initializeObject IN hDesk.
  
    HIDE FRAME frConnect.
  
    SESSION:SET-WAIT-STATE("").
  
    RUN process_user_preferences.

    /*
     Run the startup procedure and wait.  If no startup procedure has been
     defined or the startup procedure cannot be found, then only run the tabletop.
     We must change the extension of the startup procedure to .r since we store them
     in the registry as .p.  Otherwise, the SEARCH will fail.
    */
    IF cStartupProc = ? OR cStartupProc = "" OR SEARCH(cStartupProc) = ? THEN
      WAIT-FOR CLOSE OF hDesk.       
    ELSE
      RUN VALUE(cStartupProc).
      
    /*
     If the user closes Progress while RTB is still running, this should log
     them out of the repository.
    */
    IF VALID-HANDLE(hDesk) THEN DO:
        
      /*
       Get the current session id (because you may have logged out and back in),
       and Logout of RTB session
      */
      PUBLISH "evRtbGetSid" (OUTPUT iSid).
      RUN rtb_logout(iSid).
        
      /*
       Make sure the Tabletop window's close event trigger runs
      */
      {rtb/admtwo/get ContainerHandle hWindow hDesk}.
      APPLY "WINDOW-CLOSE":U TO hWindow.

    END.
  
    /*
     Bye bye...
    */
    RUN rtb_end_session.

  END. /* Mprogress-running */
    
  ELSE DO:
    RUN initializeObject IN hDesk.
    HIDE FRAME frConnect.
    SESSION:SET-WAIT-STATE("").
  END.

END. /* lOk */
ELSE DO:
  RUN rtb_logout(iSid).
  RUN rtb_end_session.
END.

/* **********************  Internal Procedures  *********************** */

PROCEDURE process_user_preferences PRIVATE:

  /*
   Determine startup procedure.
  */
  ASSIGN
    cStartupProc = DYNAMIC-FUNCTION('fnRtbGetRegistryValue':U,"Startup":U,"StartupProcedure":U)
    cStartupProc = DYNAMIC-FUNCTION('fnRtbStripExtension':U,cStartupProc) + ".r":U.
    
  /*
   Create Database Connection preference if it doesn't exist.
  */
  IF DYNAMIC-FUNCTION('fnRtbGetRegistryValue':U,"Databases":U,"ConnectWorkspaceDBs":U) = "":U THEN
    DYNAMIC-FUNCTION('fnRtbSetRegistryValue':U,"Databases":U,"ConnectWorkspaceDBs":U,"Yes":U).

  RETURN.
  
END PROCEDURE.

PROCEDURE rtb_end_session PRIVATE:
  
  PUBLISH {&RTB-USER-EVENT}
    (INPUT {&SESSION-SHUTDOWN},
     INPUT STRING(iSid),
     INPUT "",
     INPUT "",
     OUTPUT lOk).

  QUIT.

END PROCEDURE.

PROCEDURE rtb_logout PRIVATE:
  
  DEFINE INPUT PARAMETER piSid AS INTEGER NO-UNDO.
  
  DEFINE VARIABLE hSdo AS HANDLE NO-UNDO.
  
  IF piSid > 0 THEN DO:
    hSdo = fnRtbStartSuperSdo("rtb_u_sess.w":U,THIS-PROCEDURE,NO).
    DYNAMIC-FUNCTION('fnRtbLogOut':U IN hSdo,piSid).
    fnRtbEndSuperProc(THIS-PROCEDURE,hSdo,YES).
  END.
  
  RETURN.

END PROCEDURE.

/******** Function implementations **********/

FUNCTION setProgressPropath RETURNS LOGICAL (INPUT pcDlcPath AS CHARACTER):
  
  DEFINE VARIABLE hSdo AS HANDLE NO-UNDO.
  
  hSdo = fnRtbStartSuperSdo ("rtb_u_sess.w":U,THIS-PROCEDURE,NO).
  DYNAMIC-FUNCTION('fnRtbSetDlcpath':U,cDlcPropath).
  fnRtbEndSuperProc(THIS-PROCEDURE,hSdo,YES).

  RETURN TRUE.

END FUNCTION.

FUNCTION setRoundtablePropath RETURNS CHARACTER():
  /*
   Set the PROPATH based on where this program was started
  */
  DEFINE VARIABLE cProcFilepath  AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cRtbDlcPropath AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cRtbPropath    AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cRtbProgPath   AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cScmPath       AS CHARACTER NO-UNDO.
  
  ASSIGN cProcFilepath = SEARCH(THIS-PROCEDURE:FILE-NAME).

  IF cProcFilepath = ? THEN DO:
    
    IF THIS-PROCEDURE:FILE-NAME = "_rtb.r" THEN
      cProcFilepath = SEARCH("./_rtb.r").
    
    IF THIS-PROCEDURE:FILE-NAME = "_rtb.p" THEN
      cProcFilepath = SEARCH("./_rtb.p").

  END.

  IF cProcFilepath = ? THEN
    cProcFilepath = THIS-PROCEDURE:FILE-NAME.

  ASSIGN
    cProcFilepath  = REPLACE(cProcFilepath,"\","/")
    cProcFilepath  = SUBSTRING(cProcFilepath,1,(MAX(1,R-INDEX(cProcFilepath,"/") - 1)))
    cRtbDlcPropath = PROPATH
    cRtbPropath    = cRtbDlcPropath.

  IF NOT CAN-DO(REPLACE(cRtbPropath, "\", "/"), cProcFilepath) THEN
    cRtbPropath = cProcFilepath + "," + cRtbPropath.

  /* If the Dynamics integration SCM directory is in the path, then add this to the PROPATH as well */
  FILE-INFO:FILE-NAME = cProcFilepath + "/scm":U. 
  cScmPath = FILE-INFO:FULL-PATHNAME.
      
  IF LENGTH(cScmPath) GT 0 THEN
    ASSIGN 
      cScmPath = cScmPath + ",":U + cScmPath + "/custom,":U + cScmPath + "/custom/rtb,":U
      cScmPath = REPLACE(cScmPath, "~\":U, "/":U). 
  ELSE
    cScmPath = "":U.
  
  ASSIGN
    cRtbPropath  = cScmPath
                 + cProcFilepath + "/rtb,"
                 + cProcFilepath + "/rtb/proxy/p,"
                 + cProcFilepath + "/pureabltv,"
                 + cRtbPropath
    cRtbProgPath = cRtbPropath
    PROPATH      = cRtbPropath.

  RETURN cRtbPropath.

END FUNCTION.

FUNCTION isRoundtableRunning RETURNS LOGICAL():
  /*
   Check to see if Roundtable is already running by stepping backward
   through the call tree. If we run out of programs before we encounter
   a program-name that doesn't contain "_rtb" (the name of this program),
   then this is the startup program of a new Progress session.
   
   Example:
   
     PROGRAM-NAME(1):  isRoundtableRunning C:/Roundtable/_rtb.p
     PROGRAM-NAME(2):  C:/Roundtable/_rtb.p
     PROGRAM-NAME(3):  ?
     
   If Roundtable is already running, PROGRAM-NAME(3) will return the
   program that ran _rtb.p and so forth. These program names will NOT
   contain "_rtb".
  */
  DEFINE VARIABLE iCnt     AS INTEGER NO-UNDO INITIAL 1.
  DEFINE VARIABLE lRunning AS LOGICAL NO-UNDO INITIAL NO.

  DO WHILE (PROGRAM-NAME(iCnt) <> ?):
    
    IF INDEX(PROGRAM-NAME(iCnt),"_rtb") = 0 THEN DO:
      lRunning = YES.                                                
      RETURN lRunning.
    END.  

    iCnt = iCnt + 1.  

  END. 

  RETURN lRunning.

END FUNCTION.
